// PPortIoD.c
//
//  Control parallel port direct I/O daemon via "tray icon"
//
//    usage: PPortIoD.exe 1
//

//#define SHOW_OPEN_CLOSE_MBOXS
//#define ONLY_SHOW_MBOX_ON_DUPLICATES
//#define REMOVE_DRIVER_ON_EXIT

unsigned char PPortIoDExe[1];
int PPortIoDExeLen = 0;

#if DBG
#define TRC(_x) OutputDebugString(TEXT(_x))
#else
#define TRC(_x)
#endif

#include "PPortIoD.h"
#include <windowsx.h>
#include <winuser.h>
#include "resource.h"

#include "../SetPrivIoXP.h"
#include "../InstDriverXP.h"
#include "../LogPrivIoXP.h"

//
// tray icon structure
//
struct _DLGITEMS {
    DWORD dwStart;
    UINT uNotify;
} g_sDlgItems[] = { {
0, IDC_NOTIFY1,},};

static HINSTANCE g_hinst = NULL;
static char *szTip = "Parallel port I/O daemon";
static char szWinName[256];
static char szRunString[256];
static char szDeviceName[256];
static int nPort;

//
// entry point
//
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{
    int stat;

    wsprintf(szRunString, "%s", lpCmdLine);
    nPort = atoi(szRunString);
    if (*szRunString == '\0' || nPort < 0) {
        MessageBox(0,
               "Lock parallel port for direct I/O access:\n"
               "\n"
               "Usage:\n"
               "\tPPortIoD.exe N\n"
               "where:\n"
               "\tN : parallel port number (1,2,...)\n"
               , "Program Usage", MB_OK | MB_ICONINFORMATION);
        PostQuitMessage(0);
    }

    MakePortNamePrivIoXP(nPort, szDeviceName, sizeof(szDeviceName));
    wsprintf(szWinName, "PPortIoD %s", szDeviceName);

    g_hinst = hInstance;

    //stat = MessageBox(0, "Testing", "Caption", MB_OKCANCEL | MB_TASKMODAL);
    //if (stat == IDCANCEL) PostQuitMessage(1);
    //stat = MessageBox(0, lpCmdLine, "Command Line", MB_OK);

    stat =
        DialogBox(hInstance, MAKEINTRESOURCE(IDD_PPORTIOD), NULL,
                  PPortIoDDlgProc);
    if (stat < 0) {
        exit(1);
    }
    return (FALSE);
}

static HANDLE bOpenDevice(char *szDeviceName)
{
    HANDLE handle;

    handle = CreateFile(szDeviceName,
                        GENERIC_READ,
                        FILE_SHARE_READ,
                        NULL,
                        OPEN_EXISTING,
                        FILE_ATTRIBUTE_NORMAL,
                        NULL);
    return handle;
}

//
//  check for duplicate window name
//
static int nDupCount = 0;
static BOOL CALLBACK EnumProc(HWND hWindow, LPARAM  lParam)
{
    char sTitle[256] ;
    DWORD Pid ;

    GetWindowThreadProcessId(hWindow, &Pid) ;
    GetWindowText(hWindow, sTitle, sizeof(sTitle) - 1) ;
    if (!strcmp(sTitle, szWinName)) {
        ++nDupCount;
    }
    return(TRUE) ;
}

static BOOL CheckForDuplicate(BOOL bAbort)
{
    nDupCount = 0;
    EnumWindows(EnumProc, 0) ;
    if (nDupCount > 1) {
        if (bAbort) {
#ifdef ONLY_SHOW_MBOX_ON_DUPLICATES
            MessageBox(0, "Duplicate Window!", szWinName, MB_OK);
#else
            PostQuitMessage(0);
#endif
        }
        return TRUE;
    } else {
        return FALSE;
    }
}

//
//  main dialog window
//
BOOL CALLBACK PPortIoDDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam,
                              LPARAM lParam)
{
    static BOOL bFirstTime = TRUE;

    switch (uMsg) {

    case WM_INITDIALOG:
        NotifyAdd(hDlg, 0);     // add tray notification icon
        if (!IsNT()) {
            szTip = "";
        }
        TrayMessage(hDlg, NIM_MODIFY, g_sDlgItems[0].uNotify,
                    LoadImage(g_hinst,
                              MAKEINTRESOURCE(IDI_PPORTIOD),
                              IMAGE_ICON, 16, 16, 0),
                    szTip);

        if (bFirstTime) {
            bFirstTime = FALSE;

            SetWindowText(hDlg, szWinName);
            CheckForDuplicate(TRUE);

#ifdef DO_NOT_INSTALL_DRIVER
            if (bOpenDevice(szDeviceName) == INVALID_HANDLE_VALUE) {
                char text[256];

                _snprintf(text, sizeof(text), 
                           "Can't open device %s!", szDeviceName);
                MessageBox(0, text, "Parallel Port Device Open Error",
                           MB_OK | MB_ICONEXCLAMATION);
                PostQuitMessage(0);
            }
#else
            GetLogPrivIoXP();   // clear logs
            if (OpenPrivIoXP(TRUE, "PPortIoD: ", TRUE, nPort, &nPort,
                                         NULL, NULL) == INVALID_HANDLE_VALUE) {
                MessageBox(0, GetLogPrivIoXP(),
                           "Parallel Port Device Open Error",
                           MB_OK | MB_ICONEXCLAMATION);
                PostQuitMessage(0);
            } else {
#ifdef SHOW_OPEN_CLOSE_MBOXS
                MessageBox(0, GetLogPrivIoXP(),
                           "Parallel Port Device Opened",
                           MB_OK);
#endif
            }
#endif
            MakePortNamePrivIoXP(nPort, szDeviceName, sizeof(szDeviceName));
            wsprintf(szWinName, "PPortIoD %s", szDeviceName);
            SetWindowText(hDlg, szWinName);
            CheckForDuplicate(TRUE);

            SetTimer(hDlg, VERIFY_RESCHANGE, VERIFY_TIMEOUT, VerifyTimerProc);
        }
        break;

    case WM_DESTROY:
        NotifyDelete(hDlg, 0);
        break;

    case WM_COMMAND:
        switch (GET_WM_COMMAND_ID(wParam, lParam)) {

        case IDCANCEL:
            //EndDialog(hDlg, TRUE);
            ShowWindow(hDlg, SW_HIDE);
            break;

        case IDABORT:
            ///ShowWindow(hDlg, SW_HIDE);
            NotifyDelete(hDlg, 0);
            UnSetPrivIoXP();  // release driver
#ifdef REMOVE_DRIVER_ON_EXIT
            GetLogPrivIoXP();   // clear logs
            if (!RemoveDriverXP(TRUE, "", DRIVER_NAME, FALSE)) {
                MessageBox(hDlg, GetLogPrivIoXP(),
                           "Parallel Port Device Close Error",
                           MB_OK | MB_ICONEXCLAMATION);
#ifdef SHOW_OPEN_CLOSE_MBOXS
            } else {
                MessageBox(hDlg, GetLogPrivIoXP(),
                           "Parallel Port Device Closed",
                           MB_OK | MB_ICONEXCLAMATION);
#endif
            }
#endif /* REMOVE_DRIVER_ON_EXIT */
            PostQuitMessage(0);
            break;

        case IDM_ABOUT:
            DialogBox(g_hinst, MAKEINTRESOURCE(IDD_ABOUT),
                      hDlg, (DLGPROC) About);
            break;

        default:
            break;
        }
        break;

    case MYWM_NOTIFYICON:
        switch (lParam) {

        case WM_LBUTTONDOWN:
        case WM_RBUTTONDOWN:
            switch (wParam) {

            case IDC_NOTIFY1:
                ShowWindow(hDlg, SW_RESTORE);
                SetForegroundWindow(hDlg);
                break;

            default:
                break;
            }

        default:
            break;
        }
        break;

    default:
        return (FALSE);
    }
    return (TRUE);
}

//
//  process "tray" messages
//
BOOL TrayMessage(HWND hDlg, DWORD dwMessage, UINT uID, HICON hIcon,
                 PSTR pszTip)
{
    BOOL res;
    NOTIFYICONDATA tnd;

    hIcon = (HICON) LoadImage(g_hinst, MAKEINTRESOURCE(IDI_PPORTIOD),
                          IMAGE_ICON, 16, 16, 0);

    tnd.cbSize = sizeof(NOTIFYICONDATA);
    tnd.hWnd = hDlg;
    tnd.uID = uID;
    tnd.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;
    tnd.uCallbackMessage = MYWM_NOTIFYICON;
    tnd.hIcon = hIcon;

    if (pszTip) {
        lstrcpyn(tnd.szTip, pszTip, sizeof(tnd.szTip));
    } else {
        lstrcpyn(tnd.szTip, szTip, sizeof(tnd.szTip));
    }
    res = Shell_NotifyIcon(dwMessage, &tnd);

    if (hIcon) {
        DestroyIcon(hIcon);
    }
    return res;
}

//
// add tray icon
//
void NotifyAdd(HWND hDlg, UINT uIndex)
{
    TrayMessage(hDlg, NIM_ADD, g_sDlgItems[uIndex].uNotify, NULL, NULL);
}

//
// remove tray icon
//
void NotifyDelete(HWND hDlg, UINT uIndex)
{
    TrayMessage(hDlg, NIM_DELETE, g_sDlgItems[uIndex].uNotify, NULL, NULL);
}

//
// process timer event
//
VOID CALLBACK VerifyTimerProc(HWND hwnd, UINT uMsg, UINT idEvent,
                              DWORD dwTime)
{
    SendMessage(hwnd, WM_COMMAND, IDCANCEL, 0);
    KillTimer(hwnd, VERIFY_RESCHANGE);
}

//
// process "about" box events
//
BOOL APIENTRY About(HWND hDlg, UINT message, UINT wParam, LONG lParam)
{
    switch (message) {
    case WM_INITDIALOG:
        return TRUE;

    case WM_COMMAND:
        if (LOWORD(wParam) == IDOK) {
            EndDialog(hDlg, TRUE);
            return TRUE;
        }
        break;

    default:
        break;
    }
    return FALSE;
}

//
// check for Windows NT (i.e., WinNT, Win2000, WinXP)
//
BOOL IsNT()
{
    OSVERSIONINFO osvi;

    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
    GetVersionEx(&osvi);
    if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) {
        return TRUE;
    } else {
        return FALSE;
    }
}
